import Vapor

final class TodoController: RouteCollection {
    func boot(router: Router) throws {
        let todos = router.grouped("todos")

        todos.get(use: self.index)
        todos.get(Todo.parameter, use: self.show)
        todos.post(use: self.create)
        todos.put(Todo.self, at: Todo.parameter, use: self.update)
        todos.delete(Todo.parameter, use: self.delete)
    }

    /// Saves a decoded `Todo` to the database.
    func create(_ req: Request) throws -> Future<Todo> {
        return try req.content.decode(Todo.self).flatMap { todo in
            return todo.save(on: req)
        }
    }

    /// Deletes a parameterized `Todo`.
    func delete(_ req: Request) throws -> Future<HTTPStatus> {
        return try req.parameters.next(Todo.self).flatMap { todo in
            return todo.delete(on: req)
        }.transform(to: .noContent)
    }

    /// Returns a list of all `Todo`s.
    func index(_ req: Request) throws -> Future<[Todo]> {
        return Todo.query(on: req).all()
    }

    /// Show a parameterized `Todo`.
    func show(_ req: Request) throws -> Future<Todo> {
        return try req.parameters.next(Todo.self)
    }

    /// Update a parameterized `Tenant`.
    func update(_ req: Request, updatedTodo: Todo) throws -> Future<Todo> {
        return try req.parameters.next(Todo.self).flatMap { todo in
            todo.title = updatedTodo.title
            return todo.save(on: req)
        }
    }
}
