import FluentPostgreSQL
import Vapor

/// A single entry of a Todo list.
final class Todo: PostgreSQLModel {
    /// The unique identifier for this `Todo`.
    var id: Int?

    /// A title describing what this `Todo` entails.
    var title: String

    var createdAt: Date?
    var updatedAt: Date?
    var deletedAt: Date?

    /// Creates a new `Todo`.
    init(id: Int? = nil, title: String) {
        self.id = id
        self.title = title
    }
}

/// Allows `Todo` to be used as a dynamic migration.
extension Todo: Migration { }

/// Allows `Todo` to be encoded to and decoded from HTTP messages.
extension Todo: Content { }

/// Allows `Todo` to be used as a dynamic parameter in route definitions.
extension Todo: Parameter { }

extension Todo: Timestampable {
    static var createdAtKey: WritableKeyPath<Todo, Date?> {
        return \.createdAt
    }

    static var updatedAtKey: WritableKeyPath<Todo, Date?> {
        return \.updatedAt
    }
}

extension Todo: SoftDeletable {
    static var deletedAtKey: WritableKeyPath<Todo, Date?> {
        return \.deletedAt
    }
}